----------
-- Locals
----------

-- Itemid => skilllevel required
local lockboxes = {
	-- PickPocket Boxes
	[16882] = 1, -- Battered Junkbox, I think you can open these with skilllevel 1
	[16883] = 50, -- Worn Junkbox, 50 minreq? not sure
	[16884] = 175, -- sturdy junkbox, info from wowwiki
	[16885] = 250, -- heavy junkbox, info from wowwiki
	[29569] = 300, -- strong junkbox, info from wowwiki
	-- Fishing Boxes
	[6354] = 1, -- Small Locked Chest, guessed the skillevel 1
	[6355] = 50, -- Sturdy Locked Chest, guessed the skillevel 50
	[13875] = 175, -- Ironbound Locked Chest, info from wowwiki
	[13918] = 250, -- Reinforced Locked Chest, info from wowwiki
	-- Lockboxes mob drops
	[4632] = 1, -- Ornate Bronze Lockbox, guessed the skillevel 1
	[4633] = 25, -- Heavy Bronze Lockbox, info from wowwiki
	[4634] = 70, -- Iron Lockbox, info from wowwiki
	[4636] = 125, -- Strong Iron Lockbox, info from wowwiki
	[4637] = 175, -- Steel Lockbox, info from wowwiki
	[4638] = 225, -- Reinforced Steel Lockbox, info from wowwiki
	[5758] = 225, -- Mithril Lockbox, info from wowwiki
	[5759] = 225, -- Thorium Lockbox, info from wowwiki
	[5760] = 225, -- Eternium Lockbox, info from wowwiki
	[31952] = 325, -- Khorium Lockbox
	-- Crafted Locks
	[6712] = 1, -- Practice Lock, created by engineers
}

local lockpicktools = {
	[5060] = true, -- Thieves' Tools
}
local pickspell = GetSpellInfo(1804)
local pickskill = GetSpellInfo(1809)

local tooltip = LockSmithTooltip
local lockpickskill = 0
local hastools
local hasspell
local busy = nil
local db
local defaults = {
	profile = {}
}
-----------
-- Addon Declaration
-----------

LockSmith = LibStub("AceAddon-3.0"):NewAddon("LockSmith", "AceEvent-3.0", "AceTimer-3.0", "AceBucket-3.0")
local addon = LockSmith

-----------
-- Addon Initialization
-----------

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("LockSmithDB", defaults, "Default")
	db = self.db
end

function addon:OnEnable()
	local class, enclass = UnitClass("player")
	if enclass == "ROGUE" then
		self:SetupButton()
		self:RegisterBucketEvent("BAG_UPDATE", 0.5, "ScanBags")
		self:RegisterEvent("LEARNED_SPELL_IN_TAB", "GetLockPickSpell")
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "ScanBags")
		self:RegisterEvent("SKILL_LINES_CHANGED", "GetLockPickLevel")
		self:RegisterEvent("UNIT_SPELLCAST_SENT", "UnlockStart")
		self:RegisterEvent("UNIT_SPELLCAST_FAILED", "UnlockStop")
		self:RegisterEvent("UNIT_SPELLCAST_STOP", "UnlockStop")
		self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "UnlockStop")
		self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "UnlockStop")
		self:RegisterMessage("LockSmith_ScanBags", "ScanBags")
		self:GetLockPickSpell()
		self:GetLockPickLevel()
			
		self:ScanBags()
	else
		DisableAddOn("LockSmith")
	end
end

function addon:OnDisable()
	self:HideButton()
end

------------
-- The gory innards
------------

local function FindLocked()
	for i=1,tooltip:NumLines() do
		if tooltip.L[i] == LOCKED then
			return true
		end
	end
	return false
end

function addon:ScanBags()
	if InCombatLockdown() then return end
	local itemid

	local boxbag, boxslot = nil,nil
	hastools = nil

	for bag = 0, 4, 1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			_,_,itemid = string.find(GetContainerItemLink(bag,slot) or "", "item:(%d+):%d+:%d+:%d+")
			itemid = tonumber(itemid)
			if itemid then
				if not boxbag and lockboxes[itemid] and lockboxes[itemid] <= lockpickskill then
					tooltip:SetBagItem( bag, slot )
					if FindLocked() then
						boxbag = bag
						boxslot = slot
					end
				elseif lockpicktools[itemid] then
					hastools = true
				end
			end
		end
	end
	if hastools and hasspell and boxbag then
		self:SetButton(boxbag, boxslot)
	else
		self:HideButton()
	end
end

function addon:GetLockPickSpell()
	local i = 1
	local enspell, spell, subspell
	hasspell = nil

	while true do
		spell, subspell = GetSpellName(i, BOOKTYPE_SPELL)
		if not spell then
			do break end
		end
		if pickspell and pickspell == spell then
			hasspell = true
		end
		i = i + 1
	end

end

function addon:GetLockPickLevel()
	local numskills = GetNumSkillLines()
	local name, rank
	for i=1,numskills do
		name, _, _, rank = GetSkillLineInfo(i)
		if name == pickskill then
			lockpickskill = rank
			return
		end
	end
	lockpickskill = 0
end

function addon:UnlockStart(event, unit, spell, rank, target )
	if unit == "player" and spell == pickspell then
		busy = true
	end
end

local function timedScan()
	addon:SendMessage("LockSmith_ScanBags")
end

function addon:UnlockStop(event, unit)
	if unit == "player" and busy then
		busy = nil
		self:ScheduleTimer(timedScan, 1)
	end
end

function addon:SetButton(bag, slot)
	if InCombatLockdown() then 
		self:HideButton()
		return
	end
	-- secure attributes!
	local button = self.button
	button:SetAttribute( "type1", "macro" )
	button:SetAttribute( "macrotext1", "/cast "..pickspell.."\n/use "..bag.." "..slot)
	button:Show()
end

function addon:HideButton()
	if not self.button then return end
	self.button:Hide()
end

function addon:SetupButton()
	if self.button then return end

	self.button = CreateFrame("Button", "LockSmithButton", UIParent, "SecureActionButtonTemplate")

	local button = self.button

	button:Hide()
	button:SetWidth( 32 )
	button:SetHeight( 32 )
	button:ClearAllPoints()
	button:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	button:EnableMouse(true)
	button:RegisterForDrag("LeftButton")
	button:RegisterForClicks("AnyUp")
	button:SetMovable(true)
	button:SetScript("OnDragStart", function() if not InCombatLockdown() and IsAltKeyDown() then button:StartMoving() end end)
	button:SetScript("OnDragStop", function() if not InCombatLockdown() then button:StopMovingOrSizing() addon:SavePosition() end end)

	button.background = button:CreateTexture( nil, "BACKGROUND")
	button.background:SetAllPoints( button )
	button.background:SetTexture( "Interface\\Icons\\Spell_Nature_MoonKey" )
	button.background:SetTexCoord( 0.07, 0.93, 0.07, 0.93)

	self:RestorePosition()
end

function addon:SavePosition()
	local f = self.button
	local s = f:GetEffectiveScale()

	self.db.profile.posx = f:GetLeft() * s
	self.db.profile.posy = f:GetTop() * s
end

function addon:RestorePosition()
	local x = self.db.profile.posx
	local y = self.db.profile.posy

	if not x or not y then return end

	local f = self.button
	local s = f:GetEffectiveScale()

	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
end
